/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 10/07/23.
//

#ifndef FGGL_COMPONENTS_CORE_SERVICES_SERVICE_HPP
#define FGGL_COMPONENTS_CORE_SERVICES_SERVICE_HPP

#include "fggl/util/safety.hpp"

#include <memory>
#include <functional>

#define FGGL_DEFINE_SERVICE(x) constexpr const static auto SERVICE_ID = fggl::services::make_name(#x);
#define FGGL_SERVICE(x) x::SERVICE_ID

#define FGGL_IMPLEMENT_SERVICE(x,...) constexpr static const auto SERVICE_API = FGGL_SERVICE(x); \
                                        constexpr static const auto SERVICE_DEPS = fggl::services::svc_extract_names<__VA_ARGS__>();

namespace fggl::services {

	template<typename T>
	struct Service {
		public:
			inline Service(std::shared_ptr<T> service) : m_service(service) {}

			inline T* operator->() {
				return m_service.operator->();
			}

			inline T* ptr() {
				return m_service.get();
			}
		private:
			std::shared_ptr<T> m_service;
	};

	using ServiceName = util::OpaqueName<std::string_view, struct ModuleServiceTag>;

	template<typename T>
	concept ServiceAPI = requires(T* type) {
		{ T::SERVICE_ID };
	};

	template<typename T>
	concept ServiceProvider = requires(T* type) {
		{ T::Service::API };
		{ T::Service::DEPS };
	};

	constexpr ServiceName make_name(const std::string_view name) {
		return ServiceName::make(name);
	}

	template<typename... T>
	constexpr auto svc_extract_names() {
		return std::array<ServiceName, sizeof...(T)>{ T::SERVICE_ID... };
	}


} // namespace fggl::services

#endif //FGGL_COMPONENTS_CORE_SERVICES_SERVICE_HPP
