/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

#include <phys/types.hpp>
#include <fggl/entity/module.hpp>

namespace fggl::phys {

	void euler(entity::EntityManager& entities, float duration) {

		auto physEntities = entities.find<math::Transform, Motion, Dynamics>();

		for ( const auto& entity : physEntities ) {
			auto& transform = entities.get<math::Transform>(entity);
			auto& motion = entities.get<Motion>(entity);
			auto& dynamics = entities.get<Dynamics>(entity);

			 transform.origin( math::add_scaled(transform.origin(), motion.velocity, duration) );
			//math::add_scaled(entity.position, entity.acceleration, duration * duration * FLOAT_HALF);

			math::vec3f const resultingAcc = dynamics.force;
			motion.velocity = math::add_scaled( motion.velocity, resultingAcc, duration );
			motion.velocity *= powf( motion.damping, duration);

			dynamics.force = math::VEC3_ZERO;
		}

	}

} // namespace fggl::phys