/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 24/06/23.
//

#include <phys/module.hpp>
#include <phys/types.hpp>

namespace fggl::phys {

	namespace {
		void build_internal(const entity::ComponentSpec & /*config*/,
							   entity::EntityManager &manager,
							   const entity::EntityID &entity,
							   services::Generator */*services*/) {
			manager.add<fggl::phys::RigidBody>(entity);
			manager.add<fggl::phys::Dynamics>(entity);
			manager.add<fggl::phys::Motion>(entity);
		}
	}

	constexpr float DELTA_60 = 1000.0F / 60.0F;
	constexpr float DELTA_TO_SECONDS = 1.F / 1000.F;

	InternalPhysicsEngine *InternalPhysicsProvider::create(entity::EntityManager *manager, entity::EntityFactory *factory) {
		factory->bind(util::make_guid("phys::Body"), build_internal);
		return new InternalPhysicsEngine(manager);
	}

	void InternalPhysics::wireServices(services::ModuleBinder &binder) {
		binder.bind<phys::PhysicsProvider>().simplyTo<InternalPhysicsProvider>().asSingleton();
	}

	void checkCollisions(entity::EntityManager& manager) {
		auto colliders = manager.find<math::Transform, RigidBody>();
		for ( auto entity : colliders ) {

		}
	}

	void InternalPhysicsEngine::step() {
		euler( *m_entities, DELTA_60 * DELTA_TO_SECONDS );
	}


	bool overlaps(const entity::EntityID& e1, math::Transform& t1, RigidBody& b1,
				  const entity::EntityID& e2, math::Transform& t2, RigidBody& b2, ContactPoint& cp) {

		auto distance = glm::distance(t1.origin(), t2.origin());
		if ( distance <= 2.0F ) {
			cp.entityA = e1;
			cp.entityB = e2;

			return true;
		}

		return false;
	}

	std::vector<ContactPoint> InternalPhysicsEngine::scanCollisions(entity::EntityID target) {
		// really dumb N^2 version, need to use acceleration structure

		auto colliders = m_entities->find<math::Transform, RigidBody>();
		std::vector<ContactPoint> contacts;

		auto targetTransform = m_entities->get<math::Transform>(target);
		auto targetBody = m_entities->get<RigidBody>(target);

		for ( const auto& entity : colliders ) {
			// cannot collide with itself
			if ( entity == target) {
				continue;
			}

			auto transform = m_entities->get<math::Transform>(entity);
			auto body = m_entities->get<RigidBody>(entity);

			// check there is an overlap
			ContactPoint contact{};
			if ( overlaps(target, targetTransform, targetBody, entity, transform, body, contact)  ) {
				contacts.push_back(contact);
			}

		}

		return contacts;
	}

	entity::EntityID InternalPhysicsEngine::raycast(math::vec3, math::vec3) {
		entity::EntityID result;
		return result;
	}

	std::vector<entity::EntityID> InternalPhysicsEngine::raycastAll(math::vec3, math::vec3) {
		return std::vector<entity::EntityID>();
	}

	std::vector<entity::EntityID> InternalPhysicsEngine::sweep(PhyShape &shape,
															   math::Transform &from,
															   math::Transform &to) {
		return std::vector<entity::EntityID>();
	}

	InternalPhysicsEngine::InternalPhysicsEngine(entity::EntityManager *entities) : m_entities(entities) {

	}

}