/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 20/11/22.
//

#include "fggl/assets/types.hpp"

namespace fggl::assets {
	auto make_asset_id_rt(const std::string &pack, const std::string &path, const std::string &view) -> AssetID {
		auto fullPath = pack + ":" + path;
		if (!view.empty()) {
			fullPath += "[" + view + "]";
		}

		#ifndef NDEBUG
		util::intern_string(fullPath.c_str());
		#endif

		auto hash = util::hash_fnv1a_64(fullPath.c_str());
		return AssetID::make(hash);
	}

	auto asset_from_user(const std::string &input, const std::string &pack) -> AssetID {

		if (input.find(':') != std::string::npos ) {
			// probably fully qualified
			#ifndef NDEBUG
			util::intern_string(input.c_str());
			#endif
			auto hash = util::hash_fnv1a_64(input.c_str());
			return AssetID::make(hash);
		}

		// probably local
		return make_asset_id_rt(pack, input);
	}

}