/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 12/07/23.
//

#include "fggl/display/glfw/module.hpp"

namespace fggl::display {

	void GLFW::wireServices(services::ModuleBinder& binder)  {
		// FIXME creating a service requires context to exist, which requires starting glfw too early
		binder.bind<WindowService>().to<glfw::WindowService, gfx::WindowGraphics>(m_context);
	}

	void GLFW::setup(services::Generator* generator) {
		// this is safe, because we know that input has already been set-up before us
		auto* input = generator->getLazy<input::Input>().ptr();
		m_context = std::make_shared<glfw::GlfwContext>(input);
	}

	void GLFW::teardown() {
		// window services no longer available
		m_context = nullptr;
	}

} // namespace glfw
