/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 15/10/22.
//

#ifndef FGGL_SCRIPT_ENGINE_H
#define FGGL_SCRIPT_ENGINE_H

#include "fggl/modules/module.hpp"

namespace fggl::script {

	class ScriptEngine {
		public:
			virtual ~ScriptEngine() = default;

			// TODO use protected virtual pattern

			// scene callbacks
			virtual void onActivate() = 0;
			virtual void onDeactivate() = 0;
			virtual void onUpdate() = 0;

			// trigger callback
			virtual void onEvent(const std::string& name) = 0;

			// run code in engine
			virtual bool run(const char* script) = 0;
			virtual bool load(const char* filename) = 0;

			virtual void setGlobal(const char* name, void* ptr) = 0;
	};

	class ScriptProvider {
		public:
			FGGL_DEFINE_SERVICE(fggl::script::service)
			virtual ScriptEngine* create() = 0;
	};

}

#endif //FGGL_SCRIPT_ENGINE_H
